'''
Created on 2020年10月18日

命令行参数，环境变量，配置文件

1. 正式开始运行之前，把所有这些配置信息合并到一起。
2. 环境变量的名字是不会变的，硬编码即可（区分大小写）
3. 配置文件位置，有默认值，但可以通过环境变量和命令行参数指定别的位置。
4. 分层
    1. 第一层，解析命令行参数，选择不同子命令运行
    2. 第二层，子命令根据命令行参数和环境变量加载配置文件，并执行实际操作
    3. 实际操作主要涉及仓库信息的增删改查，这些封装到类中

@author: chenx
'''
import os
import configparser
import requests
from pickle import NONE

_config_file = 'gh.conf'
_config_file_path = os.path.abspath(os.path.expanduser(
    os.path.join('~', '.config', _config_file)))

def load_config(file):
    config = configparser.ConfigParser()
    config.read(file, encoding='UTF-8')
    return config

def _get_token():
    
    if (token := os.getenv('GH_TOKEN', None)) != None:
        return token
    config = load_config(_config_file_path)
    main = config['main']
    return main['token']

class BaseCmd:
    base_url = 'https://api.github.com'
    __version_header = {'Accept': 'application/vnd.github.v3+json'}
    
    def __init__(self, user_name):
        self.user_name = user_name
        self.__token = None
        self.__no_token = False
        
    def __call__(self):
        raise NotImplementedError()
    
    @property
    def token(self):
        if self.__token is None and not self.__no_token:
            token = _get_token()
            if token is None: self.__no_token = True
            else: self.__token = token
            
        return self.__token

    @property
    def headers(self):
        auth_header = {}
        if self.token is not None:
            auth_header =   {'Authorization': 'token ' + self.token }

        return { ** self.__version_header, **auth_header }
    @property
    def is_authorized(self):
        ''' 是否有口令
        '''
        return self.token is not None
    
    @property
    def name(self):
        ''' 子命令名称
        '''
        raise NotImplementedError()


class CreateRepo(BaseCmd):
    '''
    post /user/repos
    Status: 201 Created
    
    OAuth
    - public_repo scope or repo scope to create a public repository
    - repo scope to create a private repository
    '''
    
    def __init__(self, user_name = None, token = None):
        super().__init__(user_name)
    
    
    def __call__(self, args):
        print('创建仓库')
        pth = '/user/repos'
        data = {
            'name':args.repo
        }
        
        headers = super().headers
        
        res = requests.post(self.base_url + pth, headers = headers, json = data)

        if res.status_code == 201:
            print('创建成功')
        else:
            print('创建失败,', res.reason)
            print(res.headers['status'])
        
    @property
    def name(self):
        ''' 子命令名称
        '''
        return 'create'
    
class DeleteRepo(BaseCmd):
    '''
    删除仓库的api
    [仓库 - GitHub Docs](https://docs.github.com/cn/free-pro-team@latest/rest/reference/repos#delete-a-repository)
    '''
    
    def __init__(self, user_name = None, token = None):
        super().__init__(user_name)
    
    
    def __call__(self, args):
        print('删除仓库')
        pth = '/repos/chenxizhan1995/' + args.repo

        headers = super().headers
        
        res = requests.delete(self.base_url + pth, headers = headers)

        if res.status_code == 204: #
            print('删除成功')
        elif res.status_code == 403:
            print('无删除权限')
        elif res.status_code == 404:
            print('仓库不存在')
        else:
            print('删除失败:' + res.reason)
    
    @property
    def name(self):
        ''' 子命令名称
        '''
        return 'delete'
    

class ListRepo(BaseCmd):
    '''
    get /user/repos
    Status: 200 OK
    '''
    def __init__(self,user_name = None, token = None):
        super().__init__(user_name)
    
    def __call__(self, args):
        print('列举仓库')
        pth = '/user/repos'

        headers = super().headers
        params = {
            'per_page':'100'
        }
        res = requests.get(self.base_url + pth, headers = headers, params = params)
        if res.status_code == 200: #
            repos = res.json()
            print('一共 %d 个仓库' %(len(repos)))
            for idx, repo in enumerate(repos, 1):
                print(idx, '-'*60)
                print('%s' %(repo['full_name']))
                if repo['private']: print('私有')
                if repo['fork']: print('fork')
                print('描述：', repo['description'])
        else:
            print('失败:' + res.reason)

    @property
    def name(self):
        ''' 子命令名称
        '''
        return 'list'
    
class ShowRepo(BaseCmd):
    '''
    
    get /repos/{owner}/{repo}
    Status: 200 OK
    
    [仓库 - GitHub Docs](https://docs.github.com/cn/free-pro-team@latest/rest/reference/repos#get-a-repository)
    
    '''
    
    def __init__(self, user_name = None, token = None):
        super().__init__( user_name)
    
    def __call__(self, args):
        import json
        pth = '/repos/' + args.repo
        
        headers = super().headers

        res = requests.get(self.base_url + pth, headers = headers)
        print(json.dumps(res.json(), indent=4))
        
    @property
    def name(self):
        ''' 子命令名称
        '''
        return 'show'
    
_list = ListRepo('list')
_create_repo_cmd = CreateRepo('create')
_show_repo_cmd = ShowRepo('show')
_delete_repo_cmd = DeleteRepo('delete')

__all__ = [_list, _create_repo_cmd, _delete_repo_cmd, _show_repo_cmd]



'''
gh -- gibhub 仓库操作

gh 是一个创建、查看仓库的命令行工具

令牌，依次从配置文件、环境变量、命令行参数读取令牌，后面的优先级更高。
配置文件：$HOME/.config/gh.conf
环境变量: GH_TOKEN
命令行参数：--token=xxx

@author:     陈希展

@copyright:  2020 chenx. All rights reserved.

@license:    GPL

@contact:    chenxizhan1995@163.com
@deffield    updated: Updated
'''

import sys
import os

from argparse import ArgumentParser
from . import api

__all__ = []
__version__ = 0.1
__date__ = '2020-10-18'
__updated__ = '2020-10-18'

DEBUG = 0
TESTRUN = 0
PROFILE = 0

class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''
    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg
    def __str__(self):
        return self.msg

def main(argv=None): # IGNORE:C0111
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by 陈希展 on %s.
  Copyright 2020 chenx. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))
    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license)
        parser.add_argument("-v", "--verbose", dest="verbose", action="count", help="set verbosity level [default: %(default)s]")
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        
        sub = parser.add_subparsers()
        
        common_args = ArgumentParser(add_help=False)
        common_args.add_argument('--token', help='OAth 令牌')
        common_args.add_argument('--repo', help='仓库名称')
        
        for cmd in api.__all__:
            tmp = sub.add_parser(cmd.name, parents=[common_args])
            tmp.set_defaults(func=cmd)
    
        # Process arguments
        args = parser.parse_args()
        if args.func:
            args.func(args)
        
        return 0
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 0
    except Exception as e:
        if DEBUG or TESTRUN:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help\n")
        return 2

if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-h")
        sys.argv.append("-v")
        sys.argv.append("-r")
    if TESTRUN:
        import doctest
        doctest.testmod()
    if PROFILE:
        import cProfile
        import pstats
        profile_filename = 'gh_profile.txt'
        cProfile.run('main()', profile_filename)
        statsfile = open("profile_stats.txt", "wb")
        p = pstats.Stats(profile_filename, stream=statsfile)
        stats = p.strip_dirs().sort_stats('cumulative')
        stats.print_stats()
        statsfile.close()
        sys.exit(0)
    sys.exit(main())
    
    


'''
Created on 2020年10月19日

@author: chenx
'''
import setuptools
with open("Readme.md", "r", encoding='UTF-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="chx-gh", 
    version="0.0.2",
    author="Xizhan Chen",
    author_email="chenxizhan1995@163.com",
    description="Github 命令行工具",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/chenxizhan1995/gh.git",
    packages=setuptools.find_namespace_packages(include=['chx.*']),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
